Imports System.Drawing
Imports Eka.Inheritance.MDIControls

Namespace CRUD
    Public Class BaseApplicationFormMain
        Inherits C1.Win.C1Ribbon.C1RibbonForm

        Private mLastActiveControl As Control

        Private mAppForms As New Dictionary(Of String, Object)

        Private mCurrentNodeTreeViewTag As TreeViewTagProperties
        Private mMenuFontSize As Single

        ''' <summary>
        ''' This will holds the list of Forms name available in the panel tree view for searching
        ''' </summary>
        ''' <remarks></remarks>
        Friend mSearchNodeList As New List(Of String)()

        ''' <summary>
        ''' NavigationPanelHelper object to draw the Left navigation panel in the main form
        ''' </summary>
        ''' <remarks></remarks>
        Private mNavigationPanelHelper As NavigationPanelHelper


#Region "Public methods"
        Public Sub New()
            'Set the form DoubleBuffered property to avoid flickering
            Me.DoubleBuffered = True

            ' This call is required by the Windows Form Designer.
            InitializeComponent()

            'initialize NavigationPanelHelper object 
            mNavigationPanelHelper = New NavigationPanelHelper(Me)

            'Open the Application in Maximized state
            Me.WindowState = FormWindowState.Normal

        End Sub
#End Region


#Region "Protected Methods"


        Protected Sub EkaInitializeForm()
            ' Add any initialization after the InitializeComponent() call.
            Try
                BuildTreeViews()
            Catch ex As Exception
                Application.Exit()
            End Try
        End Sub


        Protected Overridable Sub EkaOLoadTreeView(ByVal TreeView As System.Windows.Forms.TreeView)
            'This method must be overriden in the descendant.
        End Sub


        Protected Sub EkaAddNodeToTreeview(ByVal TreeView As Windows.Forms.TreeView, ByVal Text As String, ByVal Description As String, ByVal AppForm As Object, Optional ByVal ID As Integer = 0, Optional ByVal TimeOut As Integer = 0, Optional ByVal IsParentNode As Boolean = False, Optional ByVal GroupType As String = "")
            Dim lNode As Windows.Forms.TreeNode = Nothing
            Dim lTreeViewTag As New TreeViewTagProperties

            If ID = 0 Then
                ID = TreeView.GetNodeCount(False) + 1
            End If

            With lTreeViewTag
                .ID = ID
                .Text = Text
                .Description = Description
                .FileName = Application.ProductName
                If TypeOf AppForm Is Form Then
                    .SubFileName = CType(AppForm, Form).Name
                Else
                    .SubFileName = Text
                End If
                .TimeOut = TimeOut
                .GroupType = GroupType
            End With

            If Not mAppForms.ContainsKey(lTreeViewTag.SubFileName) Then
                mAppForms.Add(lTreeViewTag.SubFileName, AppForm)
            End If

            ' Create Tree View Node
            lNode = BuildTreeViewNode(lTreeViewTag, IsParentNode)

            ' Add TreeNode to TreeView
            TreeView.Nodes.Add(lNode)
        End Sub

        Protected Function EkaMenuContainsKey(ByVal NodeName As String) As Boolean
            Return mAppForms.ContainsKey(NodeName)
        End Function

        Protected Function EkaGetFormToOpen(ByVal NodeName As String) As Object
            Return mAppForms(NodeName)
        End Function

        Protected Overridable Sub EkaOOpenForm(ByVal NodeName As String, ByVal NodeText As String, ByVal TagProperties As TreeViewTagProperties)
            If EkaMenuContainsKey(NodeName) Then
                'Either System.Type that implements IBaseSearch passed in or a class reference that implements IBaseSearch passed in
                If (TypeOf EkaGetFormToOpen(NodeName) Is Type AndAlso DirectCast(Me.EkaGetFormToOpen(NodeName), Type).GetInterface("IBaseSearch", True) IsNot Nothing) _
                OrElse TypeOf EkaGetFormToOpen(NodeName) Is IBaseSearch Then
                    Dim lfrmToOpen As IBaseSearch

                    'We have a System.Type provided as our form to open, create a new instance of it
                    If TypeOf EkaGetFormToOpen(NodeName) Is Type Then
                        lfrmToOpen = CType(Activator.CreateInstance(DirectCast(Me.EkaGetFormToOpen(NodeName), Type)), IBaseSearch)

                    Else 'We have an instance of an object or a class reference provided, create an(other) instance it
                        lfrmToOpen = CType(Activator.CreateInstance(DirectCast(Me.EkaGetFormToOpen(NodeName), Object).GetType), IBaseSearch)
                    End If

                    lfrmToOpen.OpenForm(Me, NodeText)


                Else 'D'OH!
                    Throw New Exception("Unsupported menu form, " & NodeText)
                End If
            End If
        End Sub

#End Region

#Region "Structures and Methods Brought From FormHelper"
        Public Structure TreeViewTagProperties
            Dim ID As Integer
            Dim Text As String
            Dim Description As String
            Dim FileName As String
            Dim SubFileName As String
            Dim TimeOut As Integer
            Dim UserPrefMenuName As String
            Dim UserPrefDisplayName As String
            Dim GroupType As String
        End Structure
#End Region

#Region "Private Methods"
        Private Sub BaseApplicationFormMain_Load(sender As Object, e As EventArgs) Handles MyBase.Load
            'Ugly way to maximize the form, it is introduced to get rid of the component one Windowstate - Maximize issue

            Me.Location = New Point(0, 0)
            Me.Size = New Size(Screen.PrimaryScreen.WorkingArea.Width, Screen.PrimaryScreen.WorkingArea.Height)
        End Sub


#End Region



#Region "Tree View Routines"

        Protected Function BuildTreeViewNode(ByVal TagProperties As TreeViewTagProperties,
                                             Optional ByVal IsParentNode As Boolean = False) As System.Windows.Forms.TreeNode
            Dim lNode As New Windows.Forms.TreeNode
            Dim lSpace As String
            Dim lchrSpace As Char

            ' Build Tree View
            With lNode
                .Name = TagProperties.SubFileName
                .ToolTipText = TagProperties.Description
                .Tag = TagProperties
                If IsParentNode Then
                    .NodeFont = New System.Drawing.Font(BasicTheme.GetBasicControlsTheme().TreeViewRootNodeInfo.FontName, BasicTheme.GetBasicControlsTheme().TreeViewRootNodeInfo.FontSize, BasicTheme.GetBasicControlsTheme().TreeViewRootNodeInfo.FontStyle, System.Drawing.GraphicsUnit.Point, CType(0, Byte))
                    .ForeColor = ColorTranslator.FromHtml(BasicTheme.GetBasicControlsTheme().TreeViewRootNodeInfo.FontColor)
                    ' Parent Node's Text need to extend with Space since there's a bug in Node that will clipped node if it's font is in bold
                    ' 'http://connect.microsoft.com/VisualStudio/feedback/ViewFeedback.aspx?FeedbackID=94354

                    ' determining how much space is required for the Node.  (if large font is used, we need more space)
                    If Not Char.TryParse(" ", lchrSpace) Then
                        lchrSpace = CChar(" ")
                    End If
                    lSpace = New String(lchrSpace, Len(TagProperties.Text) + 1)
                    .Text = TagProperties.Text & lSpace

                    ' show as Folder
                    .ImageKey = "Close"
                    .SelectedImageKey = "Open"
                Else
                    .Text = TagProperties.Text
                    .NodeFont = New System.Drawing.Font(BasicTheme.GetBasicControlsTheme().TreeViewNodeInfo.FontName, BasicTheme.GetBasicControlsTheme().TreeViewNodeInfo.FontSize, BasicTheme.GetBasicControlsTheme().TreeViewNodeInfo.FontStyle, System.Drawing.GraphicsUnit.Point, CType(0, Byte))
                    .ForeColor = ColorTranslator.FromHtml(BasicTheme.GetBasicControlsTheme().TreeViewNodeInfo.FontColor)
                    TagProperties.UserPrefMenuName = TagProperties.SubFileName
                    TagProperties.UserPrefDisplayName = TagProperties.Text
                    'Add the node text to searh list
                    If Not mSearchNodeList.Contains(TagProperties.Text) Then
                        mSearchNodeList.Add(TagProperties.Text)
                    End If

                End If

                If TagProperties.Description.Trim().Length > 0 Then
                    .ToolTipText = If(TagProperties.Description.EndsWith("."), TagProperties.Description, TagProperties.Description + ".")
                End If

                .Tag = TagProperties
            End With

            Return lNode
        End Function
#End Region

#Region "Delegated Tree View Routines"


        Private Sub TreeView_NodeMouseClick(ByVal sender As Object, ByVal e As TreeNodeMouseClickEventArgs)
            If e.Button = Windows.Forms.MouseButtons.Left AndAlso e.Node.Nodes.Count = 0 Then
                Dim lTagProperties As TreeViewTagProperties = CType(e.Node.Tag, TreeViewTagProperties)

                EkaOOpenForm(e.Node.Name, e.Node.Text, lTagProperties)
            End If

        End Sub


#End Region

#Region "New Navigation Panel code"
        Friend Sub BuildTreeView(TreeView As CustomTreeView)
            ' Build Tree View, Name Parameters will be used to assign the TreeView's Name.
            Dim ltv As New Windows.Forms.TreeView

            With TreeView
                .ImageKey = "Document"  '"BookClose"
                .HotTracking = True
                .ShowNodeToolTips = True       ' currently bold text now...since we sent font to be....so disabled tooltip for now.

                '' to fixed the font bold text clipping issue, set font to bold first and unset them later
                ''http://connect.microsoft.com/VisualStudio/feedback/ViewFeedback.aspx?FeedbackID=94354

                '' Set up Tree View's Event
                If mMenuFontSize <= 0 Then
                    mMenuFontSize = 9
                End If
                .Font = New System.Drawing.Font(BasicTheme.GetBasicControlsTheme().TreeViewNodeInfo.FontName, BasicTheme.GetBasicControlsTheme().TreeViewNodeInfo.FontSize, BasicTheme.GetBasicControlsTheme().TreeViewNodeInfo.FontStyle, System.Drawing.GraphicsUnit.Point, CType(0, Byte))
                .ForeColor = ColorTranslator.FromHtml(BasicTheme.GetBasicControlsTheme().TreeViewNodeInfo.FontColor)
                AddHandler .NodeMouseClick, AddressOf TreeView_NodeMouseClick
            End With
        End Sub

        Private Sub BuildTreeViews()

            mMenuFontSize = Font.Size

            EkaOLoadTreeView(ctrStandardTreeView)

        End Sub

#End Region

    End Class
End Namespace