﻿Imports Eka.Inheritance.FormsV2.CRUD
Imports System.Drawing
Imports Eka.Inheritance.MDIControls

Public Class NavigationPanelHelper
    Implements IDisposable

#Region "Member variables"
    '    ''' <summary>
    '    ''' Track whether Dispose has been called. 
    '    ''' </summary>
    '    ''' <remarks></remarks>
    Private mIsDisposed As Boolean = False

    ''' <summary>
    ''' variable to holds the BaseApplicationFormMain object for accessing the required object to 
    ''' initialize the Left Navigation panel and theme
    ''' </summary>
    ''' <remarks></remarks>
    Private mBaseForm As BaseApplicationFormMain

    ''' <summary>
    ''' set the text for the Standard Tree View Header text in the left navigation panel
    ''' </summary>
    ''' <remarks></remarks>
    Private Const mStandardHeaderText = "STANDARD"

    Private Const mPanelHeaderHeight = 21
#End Region

#Region "Constructor"
    Public Sub New(BaseApplicationFormMain As BaseApplicationFormMain)
        mBaseForm = BaseApplicationFormMain

        mBaseForm.ctrC1StatusBar.Dock = DockStyle.Bottom

        ApplyNavigatinPanelTheme()

        ApplyLineTheme()
        ApplyStandardPanelTheme()

        ApplyStandardTreeViewTheme()

        mBaseForm.BuildTreeView(mBaseForm.ctrStandardTreeView)

        'Set the Panel width to trigger the event and set all the controls width properly
        mBaseForm.ctrNavigationPanel.Width = mBaseForm.ctrNavigationPanel.Width + 1

        'Set the MDIClient Properties
        Dim lMdiClientController = New MdiClientController(BaseApplicationFormMain)
        lMdiClientController.BorderStyle = BorderStyle.None
        lMdiClientController.BackColor = Color.White
    End Sub
#End Region

#Region "Method"

    ''' <summary>
    ''' Load the Navigation theme from the control dll and apply it to the 
    ''' Navigation panel
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub ApplyNavigatinPanelTheme()
        mBaseForm.ctrNavigationPanel.ToggleButtonPosition = CollasibleButtonPosition.Right
        mBaseForm.ctrNavigationPanel.ToggleButtonState = ToggleButtonState.Expanded
        mBaseForm.ctrNavigationPanel.Dock = DockStyle.Left
        mBaseForm.ctrNavigationPanel.Width = CInt(mBaseForm.Width * 0.25)
        mBaseForm.AutoSize = False

        mBaseForm.ctrNavigationPanel.AllowResize = True
        mBaseForm.ctrNavigationPanel.PanelResizeOption = New Eka.Inheritance.MDIControls.ResizeOptions(False, True, False, False, False, False)
        mBaseForm.ctrNavigationPanel.MinimumSize = New Size(mBaseForm.ctrNavigationPanel.AllowedMinimumSize.Width, mBaseForm.ctrNavigationPanel.AllowedMinimumSize.Height + 5)
        mBaseForm.ctrNavigationPanel.BorderThickness = New PanelBorderThickness(1, 1, 4, 1)

        mBaseForm.ctrNavigationPanel.BackColor = Color.White
        mBaseForm.ctrNavigationPanel.BorderStyle = BorderStyle.None
        mBaseForm.ctrNavigationPanel.BorderColor = ColorTranslator.FromHtml(BasicTheme.GetBasicControlsTheme().NavigationPanelInfo.BorderColor)
        mBaseForm.ctrNavigationPanel.DrawOrientationText = True
        mBaseForm.ctrNavigationPanel.DrawBorder = BasicTheme.GetBasicControlsTheme().NavigationPanelInfo.DrawBorder
        mBaseForm.ctrNavigationPanel.OrientationText = "Forms Navigation"
    End Sub

    ''' <summary>
    ''' Resize the divider line between Favorite and Standard Collapsible Panel
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub ApplyLineTheme()
        mBaseForm.ctrEkaCollapsibleLine.Dock = DockStyle.None
        mBaseForm.ctrEkaCollapsibleLine.PanelHeaderText = ""
        mBaseForm.ctrEkaCollapsibleLine.Location = New Point(mBaseForm.ctrFavoriteCollapsiblePanel.Location.X, mBaseForm.ctrFavoriteCollapsiblePanel.Location.Y + mBaseForm.ctrFavoriteCollapsiblePanel.Height + 4)
        mBaseForm.ctrEkaCollapsibleLine.Height = 1
        mBaseForm.ctrEkaCollapsibleLine.Width = GetNavigationPanelChildWidth() - 1
        mBaseForm.ctrEkaCollapsibleLine.BorderStyle = BorderStyle.None
        mBaseForm.ctrEkaCollapsibleLine.GapBetweenPanels = 2
        mBaseForm.ctrEkaCollapsibleLine.NextPanel = mBaseForm.ctrStandardEkaCollapsible
    End Sub

    ''' <summary>
    ''' Resize the Standard Collapsible panel according to the size of its parent (Left Navigation Panel)
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub ApplyStandardPanelTheme()
        mBaseForm.ctrStandardEkaCollapsible.PanelHeaderText = mStandardHeaderText
        mBaseForm.ctrStandardEkaCollapsible.Dock = DockStyle.None
        mBaseForm.ctrStandardEkaCollapsible.PanelHeaderHeight = mPanelHeaderHeight
        mBaseForm.ctrStandardEkaCollapsible.Location = New Point(mBaseForm.ctrSearchBox.Location.X, mBaseForm.ctrEkaCollapsibleLine.Location.Y + mBaseForm.ctrEkaCollapsibleLine.Height + 2)
        mBaseForm.ctrStandardEkaCollapsible.Width = GetNavigationPanelChildWidth()
        mBaseForm.ctrStandardEkaCollapsible.GapBetweenPanels = 1
        mBaseForm.ctrStandardEkaCollapsible.BorderStyle = BorderStyle.None
        mBaseForm.ctrStandardEkaCollapsible.Font = New System.Drawing.Font(BasicTheme.GetBasicControlsTheme().CollapsiblePanelInfo.FontName, BasicTheme.GetBasicControlsTheme().CollapsiblePanelInfo.FontSize, BasicTheme.GetBasicControlsTheme().CollapsiblePanelInfo.FontStyle, System.Drawing.GraphicsUnit.Point, CType(0, Byte))
        mBaseForm.ctrStandardEkaCollapsible.ForeColor = ColorTranslator.FromHtml(BasicTheme.GetBasicControlsTheme().CollapsiblePanelInfo.FontColor)
        mBaseForm.ctrStandardEkaCollapsible.BorderColor = ColorTranslator.FromHtml(BasicTheme.GetBasicControlsTheme().CollapsiblePanelInfo.BorderColor)
        mBaseForm.ctrStandardEkaCollapsible.DrawBorder = BasicTheme.GetBasicControlsTheme().CollapsiblePanelInfo.DrawBorder
        mBaseForm.ctrStandardEkaCollapsible.PanelHeaderBackGroundColor = Color.White
        mBaseForm.ctrStandardEkaCollapsible.NextPanel = Nothing

        SetStandardTreeviewHeight()
    End Sub

    ''' <summary>
    ''' Resize the Standard Tree View according to the size of its parent (Favorite Collapsible Panel)
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub ApplyStandardTreeViewTheme()
        mBaseForm.ctrStandardTreeView.Dock = DockStyle.None
        mBaseForm.ctrStandardTreeView.BorderStyle = BorderStyle.None
        SetStandardTreeviewHeight()
        mBaseForm.ctrStandardTreeView.Width = mBaseForm.ctrStandardEkaCollapsible.Width - 2
        mBaseForm.ctrStandardTreeView.Location = New Point(1, mBaseForm.ctrStandardEkaCollapsible.PanelHeaderHeight + 4)
        mBaseForm.ctrStandardTreeView.Indent = 19
        mBaseForm.ctrStandardTreeView.Font = New System.Drawing.Font(BasicTheme.GetBasicControlsTheme().TreeViewNodeInfo.FontName, BasicTheme.GetBasicControlsTheme().TreeViewNodeInfo.FontSize, BasicTheme.GetBasicControlsTheme().TreeViewNodeInfo.FontStyle, System.Drawing.GraphicsUnit.Point, CType(0, Byte))
        mBaseForm.ctrStandardTreeView.ForeColor = ColorTranslator.FromHtml(BasicTheme.GetBasicControlsTheme().TreeViewNodeInfo.FontColor)
        mBaseForm.ctrStandardTreeView.BackColor = Color.White
        mBaseForm.ctrStandardTreeView.TabIndex = 2
    End Sub



    ''' <summary>
    ''' Adjust the Standard tree view height and location in the left Navigation Panel
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub SetStandardTreeviewHeight()
        If mBaseForm.ctrStandardEkaCollapsible.ToggleButtonState = ToggleButtonState.Collapsed Then
            'do nothing
            Return
        End If

        mBaseForm.ctrStandardEkaCollapsible.Height = mBaseForm.ctrNavigationPanel.Height - mBaseForm.ctrStandardEkaCollapsible.Location.Y - 2
        mBaseForm.ctrStandardTreeView.Height = mBaseForm.ctrStandardEkaCollapsible.Height - mBaseForm.ctrStandardEkaCollapsible.PanelHeaderHeight - 4
    End Sub


#End Region

#Region "Supported method to draw left Navigation Panel"

    ''' <summary>
    ''' returns the required size (width) of the child control
    '''  which has to be placed in the Panel
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function GetNavigationPanelChildWidth() As Integer
        Return mBaseForm.ctrNavigationPanel.Width - mBaseForm.ctrNavigationPanel.Padding.Left - mBaseForm.ctrNavigationPanel.Padding.Right - 3 - mBaseForm.ctrNavigationPanel.BorderThickness.RightBorderThickness
    End Function


#End Region

#Region "IDisposable"

    ''' <summary>
    ''' Do not make this method virtual. so that a derived class should not be able to override this method.
    ''' </summary>
    ''' <remarks></remarks>
    Public Overloads Sub Dispose() Implements IDisposable.Dispose
        Dispose(True)
        ' This object will be cleaned up by the Dispose method.Therefore, you should call GC.SupressFinalize to 
        ' take this object off the finalization queue  and prevent finalization code for this object from executing a second time.
        GC.SuppressFinalize(Me)
    End Sub

    ''' <summary>
    ''' Dispose(bool disposing) executes in two distinct scenarios. If disposing equals true, the method has been called directly 
    ''' or indirectly by a user's code. Managed and unmanaged resources can be disposed. If disposing equals false, the method has been called by the  
    ''' runtime from inside the finalizer and you should not reference  other objects. Only unmanaged resources can be disposed. 
    ''' </summary>
    ''' <param name="Disposing"></param>
    ''' <remarks></remarks>
    Protected Overridable Overloads Sub Dispose(ByVal Disposing As Boolean)
        ' Check to see if Dispose has already been called. 
        If Not mIsDisposed Then
            ' If disposing equals true, dispose all managed  and unmanaged resources. 
            If Disposing Then
                ' Dispose managed resources.
                'mBaseForm = Nothing
            End If

            ' Dispose Un-managed resources here if anything

            ' Note disposing has been done.
            mIsDisposed = True
        End If
    End Sub

    ''' <summary>
    ''' This finalizer will run only if the Dispose method  does not get called. It gives your base class the opportunity to finalize.
    ''' Do not provide finalize methods in types derived from this class.  
    ''' </summary>
    ''' <remarks></remarks>
    Protected Overrides Sub Finalize()
        ' Calling Dispose(false) is optimal in terms of readability and maintainability.
        Dispose(False)
        MyBase.Finalize()
    End Sub

#End Region
End Class

