﻿using System;
using System.Windows.Forms;
using System.Runtime.InteropServices;

namespace Eka.Inheritance.MDIControls
{
    /// <summary>
    /// Class to create the customize textbox for search
    /// This will having floating text capability at the begining.
    /// </summary>
    public partial class CustomTextBox : TextBox
    {
        #region Member variables
        /// <summary>
        /// PInvoke method to send message
        /// </summary>
        /// <param name="hWnd"></param>
        /// <param name="msg"></param>
        /// <param name="wp"></param>
        /// <param name="lp"></param>
        /// <returns></returns>
        [DllImport("user32.dll", EntryPoint = "SendMessageW")]
        private static extern IntPtr SendMessage(IntPtr hWnd, int msg, IntPtr wp, IntPtr lp);

        // A delegate type for hooking up change notifications.
        public delegate void SearchTextHandler(SearchTextArgs eventArgs);

        // An event that clients can use to be notified whenever the
        // elements of the list change.
        public event SearchTextHandler SearchTextChanged;

        /// <summary>
        /// Member variable to set the floating text
        /// </summary>
        private string mFloatingText = "Enter search text here";

        #endregion

        #region Constructor
        /// <summary>
        /// Construction method
        /// </summary>
        public CustomTextBox()
        {
            InitializeComponent();

            this.Font = new System.Drawing.Font("Microsoft Sans Serif", 11F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
        }
        #endregion

        #region properties
        /// <summary>
        /// Get/Set Floating Text for the control
        /// </summary>
        public string FloatingText
        {
            get
            {
                return mFloatingText; 
            }

            set
            {
                mFloatingText = value;
                updateFloatingText();
            }
        }
        #endregion

        #region Methods
        /// <summary>
        /// Method to set the cotrol floating text by sending PInvoke message
        /// </summary>
        private void updateFloatingText()
        {
            if (!IsHandleCreated || string.IsNullOrEmpty(mFloatingText))
                return;

            IntPtr lMem = Marshal.StringToHGlobalUni(mFloatingText);
            SendMessage(Handle, 0x1501, (IntPtr)1, lMem);
            Marshal.FreeHGlobal(lMem);
        }

        /// <summary>
        /// Override the paint method for drawing the border
        /// </summary>
        /// <param name="e"></param>
        protected override void OnPaint(PaintEventArgs e)
        {
            base.OnPaint(e);
            //TODO: for border painting if required
        }

        /// <summary>
        /// Override the OnHandleCreated method to send message for binding the
        /// floating text to the control
        /// </summary>
        /// <param name="e"></param>
        protected override void OnHandleCreated(EventArgs e)
        {
            base.OnHandleCreated(e);
            updateFloatingText();
        }
       
        protected override void OnKeyDown(KeyEventArgs e)
        {
            base.OnKeyDown(e);

            if (e.KeyCode == Keys.Enter && this.Text.Trim().Length >0)
            {
                RaiseSearchTextSelected();
            }
        }
        protected virtual void RaiseSearchTextSelected()
        {
            if (SearchTextChanged == null)
                return;
                
            SearchTextArgs lSearchTextArgs = new SearchTextArgs();                
            lSearchTextArgs.SearchText = this.Text;                
            SearchTextChanged(lSearchTextArgs);
        }
       
        #endregion
    }
}