﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.ComponentModel.Design;
using System.Runtime.InteropServices;

namespace Eka.Inheritance.MDIControls
{
    /// <summary>
    /// 
    /// </summary>
    [DesignTimeVisible(true)]
    [Category("Containers")]
    [Description("Navigation Panel")]
    [Designer("System.Windows.Forms.Design.ParentControlDesigner, System.Design", typeof(IDesigner))]
    public partial class NavigationPanel : UserControl
    {
        #region Member variables and constant

        /// <summary>
        /// Memeber variable to holds the Controls Border color (#666666)
        /// </summary>
        private Color mBorderColor = Color.LightGray; //Color.FromArgb(((int)(((byte)(102)))), ((int)(((byte)(102)))), ((int)(((byte)(102)))));
        
        
        /// <summary>
        /// Member variable to holds the border style of the control.
        /// Default: None
        /// </summary>
        private ButtonBorderStyle mButtonBorderStyle = ButtonBorderStyle.Solid;

        /// <summary>
        /// constant to define the Gap(no of pixel) between label text and image icon
        /// </summary>
        private const int mGapBetweenLabelAndToggleButton = 3;

        /// <summary>
        /// Member variables to hoods Collpsing toggle state
        /// </summary>
        private ToggleButtonState mToggleButtonState;

        /// <summary>
        /// Member variable to holds the Collapsing button position
        /// </summary>
        private CollasibleButtonPosition mToggleButtonPosition;

        /// <summary>
        /// Member variable to holds the control height in collapse/expand state
        /// </summary>
        private Size mControlsSize = new Size();
        

        /// <summary>
        /// Member variable to holds the Draw border value
        /// </summary>
        private bool mDrawBorder = true;

        /// <summary>
        /// draw orientation Text whenever the panel is in collapsed state
        /// </summary>
        private bool mDrawOrientationText = true;

        /// <summary>
        /// Member variable to set the Orientatio ntext
        /// </summary>
        private string mOrientationText = "Navigation Panel";

        /// <summary>
        /// Member variable to set orientation text fore color
        /// </summary>
        private Color mOrientationFontColor = ColorTranslator.FromHtml("#333333");        

        #region for Control Resizing and moving
        /// <summary>
        /// Initialize the Panel Resize options all to False
        /// </summary>
        private ResizeOptions mPanelResizeOption = new ResizeOptions(false, false, false, false, false, false);

        /// <summary>
        /// Member variable for Panel border thickness
        /// </summary>
        private PanelBorderThickness mPanelBorderThickness = new PanelBorderThickness(1, 1, 1, 1);

        /// <summary>
        /// Member variable to track control is moving or not
        /// </summary>
        private bool mMoving;
        /// <summary>
        /// member variable to track the cursor Strting point
        /// </summary>
        private Point mCursorStartPoint;
        /// <summary>
        /// Member variable to track, that the mouse move is internal
        /// </summary>
        private bool mMoveIsInternal;
        /// <summary>
        /// Member variable to track theat the control is resizing
        /// </summary>
        private bool mResizing;
        /// <summary>
        /// Memebr variable to track the control starting size
        /// </summary>
        private Size mControlStartSize;

        /// <summary>
        /// Member variable to set the panel right hand side coloer
        /// </summary>
        private Color mRightBorderColor = System.Drawing.ColorTranslator.FromHtml("#CCE7FF");
        
        #endregion

        private const int TV_FIRST = 0x1100;
        private const int TVM_SETBKCOLOR = TV_FIRST + 29;
        private const int TVM_SETEXTENDEDSTYLE = TV_FIRST + 44;
        private const int TVS_EX_DOUBLEBUFFER = 0x0004;
        
        /// <summary>
        /// Win API to set DoubleBuffered = true for the treeview to avoid flickring
        /// </summary>
        /// <param name="hWnd"></param>
        /// <param name="msg"></param>
        /// <param name="wParam"></param>
        /// <param name="lParam"></param>
        /// <returns></returns>
        [DllImport("user32.dll")]
        public static extern IntPtr SendMessage(IntPtr hWnd, int msg, IntPtr wParam, IntPtr lParam);
        #endregion

        #region Ctor
        public NavigationPanel()
        {
            InitializeComponent();

            // Enable default double buffering processing
            SetStyle(ControlStyles.OptimizedDoubleBuffer | ControlStyles.AllPaintingInWmPaint, true);

            // Disable default CommCtrl painting on non-Vista systems
            if (!IsWinVista)
                SetStyle(ControlStyles.UserPaint, true);  
             
            //Initialize default controls properties
            InitializeDefaultControlsTheme();
        }
        #endregion

        #region Properties

        internal bool MouseIsInLeftEdge { get; set; }
        internal bool MouseIsInRightEdge { get; set; }
        internal bool MouseIsInTopEdge { get; set; }
        internal bool MouseIsInBottomEdge { get; set; }
        internal ControlOperation OperationType { get; set; }

        /// <summary>
        /// Get/Set properties for Collapsible button Positin
        /// </summary>
        [Description("Gets or sets properties for Collapsible button Positin")]
        public CollasibleButtonPosition ToggleButtonPosition
        {
            get
            { 
                return mToggleButtonPosition; 
            }

            set
            {
                if (value == mToggleButtonPosition)
                    return;

                mToggleButtonPosition = value;
                ctrToggleButtonPicturebox.Location = GetToggleButtonLocation();
                Invalidate();
            }
        }

        /// <summary>
        /// Get/Set Panel right hand side border color
        /// </summary>
        public Color RightBorderColor {
            get {
                return mRightBorderColor;
            }
            set {
                mRightBorderColor = value;
            }
        }
       
        /// <summary>
        /// Get/Set properties for Panel header text
        /// </summary>
        [Category("Eka Navigation Panel")]
        [Description("Gets or set sproperties for button toggle state")]
        public ToggleButtonState ToggleButtonState
        {
            get
            { 
                return this.mToggleButtonState;
            }

            set
            {
                if (value == this.mToggleButtonState)
                    return;

                OnToggleButtonMouseClick();
            }
        }

        /// <summary>
        /// Get/Set properties for ButtonBorderStyle
        /// </summary>
        public ButtonBorderStyle ButtonBorderStyle
        {
            get
            { 
                return mButtonBorderStyle; 
            }

            set 
            { 
                mButtonBorderStyle = value;
                Invalidate();
            }
        }

        /// <summary>
        /// Get/Set the border color
        /// </summary>
        public Color BorderColor
        {
            get 
            { 
                return mBorderColor;
            }

            set 
            { 
                mBorderColor = value;
                Invalidate(); 
            }            
        }

        /// <summary>
        /// Get/Set the border color
        /// </summary>
        public bool DrawBorder
        {
            get
            { 
                return mDrawBorder;
            }

            set 
            {
                mDrawBorder = value; 
                Invalidate();
            }
        }

        /// <summary>
        /// Get/Set the mOrientation Text
        /// </summary>
        public string OrientationText
        {
            get 
            { 
                return mOrientationText; 
            }

            set 
            {
                mOrientationText = value; 
                Invalidate(); 
            }
        }

        /// <summary>
        /// Get/Set the mOrientation Text to be draw or not
        /// </summary>
        public bool DrawOrientationText
        {
            get
            {
                return mDrawOrientationText;
            }

            set
            {
                mDrawOrientationText = value;
                Invalidate();
            }
        }

        /// <summary>
        /// 
        /// </summary>
        public bool AllowResize
        {
            get;
            set;
        }

        /// <summary>
        /// Get properties to get the minimum size of the control for resizing.
        /// </summary>
        public Size AllowedMinimumSize {
            get {
                return new Size(ctrToggleButtonPicturebox.Width + 2, ctrToggleButtonPicturebox.Height + 2);
            }
        }

        /// <summary>
        /// Get/Set properties for panel resize options.
        /// </summary>
        [Category("Collapsible Panel")]
        [Description("Gets or set properties for panel resizing.")]
        public ResizeOptions PanelResizeOption {
            get {
                return this.mPanelResizeOption;
            }
            set {
                this.mPanelResizeOption = value;
                Invalidate();
            }
        }

        /// <summary>
        /// 
        /// </summary>
        [Category("Collapsible Panel")]
        [Description("Gets or set border thickness.")]
        public PanelBorderThickness BorderThickness {
            get {                 
                return this.mPanelBorderThickness;
            }
            set {
                this.mPanelBorderThickness = value;
                ctrToggleButtonPicturebox.Location = GetToggleButtonLocation();
                Invalidate();
            }
        }
        #endregion

        #region Private Methods
        internal void Register(Control control)
        {
            Register(control, control);
        }

        internal void Register(Control control, Control container)
        {
            mMoving = false;
            mResizing = false;
            mMoveIsInternal = false;
            mCursorStartPoint = Point.Empty;
            MouseIsInLeftEdge = false;
            MouseIsInLeftEdge = false;
            MouseIsInRightEdge = false;
            MouseIsInTopEdge = false;
            MouseIsInBottomEdge = false;
            OperationType = ControlOperation.Resize;
            control.MouseDown += (sender, e) => StartMovingOrResizing(control, e);
            control.MouseUp += (sender, e) => StopDragOrResizing(control);
            control.MouseMove += (sender, e) => MoveControl(container, e);            
        }

        private void UpdateMouseEdgeProperties(Control control, Point mouseLocationInControl)
        {
            if (mToggleButtonState == ToggleButtonState.Collapsed || !AllowResize)
                return;

            if (OperationType == ControlOperation.Move)
                return;

            MouseIsInLeftEdge = Math.Abs(mouseLocationInControl.X) <= 5;
            MouseIsInRightEdge = Math.Abs(mouseLocationInControl.X - control.Width) <= 5;
            MouseIsInTopEdge = Math.Abs(mouseLocationInControl.Y) <= 5;
            MouseIsInBottomEdge = Math.Abs(mouseLocationInControl.Y - control.Height) <= 5;
        }

        private void UpdateMouseCursor(Control lControl)
        {          
            if (mToggleButtonState == ToggleButtonState.Collapsed || !AllowResize)
                return;

            if (OperationType == ControlOperation.Move)
                return;

            if (MouseIsInLeftEdge) {
                lControl.Cursor = GeLeftEdgeMouseCursor();
            }
            else if (MouseIsInRightEdge) {
                lControl.Cursor = GeRightEdgeMouseCursor();
            }
            else if (MouseIsInTopEdge) {
                lControl.Cursor = GetTopEdgeCursor();
            }
            else if (MouseIsInBottomEdge) {
                lControl.Cursor = GetBottomEdgeCursor();
            }
            else {
                //Finally it is the normal cursor
                lControl.Cursor = Cursors.Default;
            }
        }

        private Cursor GeLeftEdgeMouseCursor() {
            Cursor lLeftEdgeCursor = Cursors.Default;

            //return normal cursor if allow resize is not enabled
            if (!AllowResize)
                return lLeftEdgeCursor;

            //return normal cursor if resizing is not allowed from left
            if (!PanelResizeOption.AllowFromLeft)
                return lLeftEdgeCursor;

            //return normal cursor if mouse in not in Left Edge
            if (!MouseIsInLeftEdge)
                return lLeftEdgeCursor;

            //Now the mouse is in left edge, we need to verify the corner edge of left side
            if (MouseIsInTopEdge && PanelResizeOption.AllowResizeFromTopEdge) {
                lLeftEdgeCursor = Cursors.SizeNWSE;
            }
            else if (MouseIsInBottomEdge && PanelResizeOption.AllowResizeFromBottomEdge) {
                lLeftEdgeCursor = Cursors.SizeNESW;
            }
            else {
                lLeftEdgeCursor = Cursors.SizeWE;
            }

            return lLeftEdgeCursor;
        }

        private Cursor GeRightEdgeMouseCursor() {
            Cursor lRightEdgeCursor = Cursors.Default;

            //return normal cursor if allow resize is not enabled
            if (!AllowResize)
                return lRightEdgeCursor;

            //return normal cursor if resizing is not allowed from left
            if (!PanelResizeOption.AllowFromRight)
                return lRightEdgeCursor;

            //return normal cursor if mouse in not in Left Edge
            if (!MouseIsInRightEdge)
                return lRightEdgeCursor;

            //Now the mouse is in left edge, we need to verify the corner edge of left side
            if (MouseIsInTopEdge && PanelResizeOption.AllowResizeFromTopEdge) {
                lRightEdgeCursor = Cursors.SizeNESW;
            }
            else if (MouseIsInBottomEdge && PanelResizeOption.AllowResizeFromBottomEdge) {
                lRightEdgeCursor = Cursors.SizeNWSE;
            }
            else {
                lRightEdgeCursor = Cursors.SizeWE;
            }

            return lRightEdgeCursor;
        }

        private Cursor GetTopEdgeCursor() {
            Cursor lTopEdgeCursor = Cursors.Default;
            //Check the mouse is in top edge or not
            if (!MouseIsInTopEdge)
                return lTopEdgeCursor;

            //return normal cursor if allow resize is not enabled
            if (!AllowResize)
                return lTopEdgeCursor;

            //return normal cursor if resizing is not allowed from left
            if (!PanelResizeOption.AllowFromTop)
                return lTopEdgeCursor;

            //Now resize is allowed from top, return the resize cursor for top edge
            lTopEdgeCursor = Cursors.SizeNS;

            return lTopEdgeCursor;
        }

        private Cursor GetBottomEdgeCursor() {
            Cursor lBottomEdgeCursor = Cursors.Default;

            //Check the mouse is in top edge or not
            if (!MouseIsInBottomEdge)
                return lBottomEdgeCursor;

            //return normal cursor if allow resize is not enabled
            if (!AllowResize)
                return lBottomEdgeCursor;

            //return normal cursor if resizing is not allowed from left
            if (!PanelResizeOption.AllowFromBottom)
                return lBottomEdgeCursor;

            //Now resize is allowed from top, return the resize cursor for top edge
            lBottomEdgeCursor = Cursors.SizeNS;

            return lBottomEdgeCursor;
        }

        private void StartMovingOrResizing(Control lControl, MouseEventArgs lEvent)
        {
            if (mToggleButtonState == ToggleButtonState.Collapsed || !AllowResize)
                return;

            if (mMoving || mResizing)
                return;

            if (OperationType != ControlOperation.Move &&
                (MouseIsInRightEdge || MouseIsInLeftEdge || MouseIsInTopEdge || MouseIsInBottomEdge))
            {
                mResizing = true;
                mControlStartSize = lControl.Size;
            }
            else if (OperationType != ControlOperation.Resize)
            {
                mMoving = true;
                lControl.Cursor = Cursors.Hand;
            }
            mCursorStartPoint = new Point(lEvent.X, lEvent.Y);
            lControl.Capture = true;
        }

        private void MoveControl(Control lControl, MouseEventArgs lEvent)
        {
            if (mToggleButtonState == ToggleButtonState.Collapsed || !AllowResize)
                return;

            if (!mResizing && !mMoving)
            {
                UpdateMouseEdgeProperties(lControl, new Point(lEvent.X, lEvent.Y));
                UpdateMouseCursor(lControl);
            }
            if (mResizing)
            {
                if (MouseIsInLeftEdge)
                {
                    if (MouseIsInTopEdge)
                    {
                        lControl.Width -= (lEvent.X - mCursorStartPoint.X);
                        lControl.Left += (lEvent.X - mCursorStartPoint.X);
                        lControl.Height -= (lEvent.Y - mCursorStartPoint.Y);
                        lControl.Top += (lEvent.Y - mCursorStartPoint.Y);
                    }
                    else if (MouseIsInBottomEdge)
                    {
                        lControl.Width -= (lEvent.X - mCursorStartPoint.X);
                        lControl.Left += (lEvent.X - mCursorStartPoint.X);
                        lControl.Height = (lEvent.Y - mCursorStartPoint.Y) + mControlStartSize.Height;
                    }
                    else
                    {
                        lControl.Width -= (lEvent.X - mCursorStartPoint.X);
                        lControl.Left += (lEvent.X - mCursorStartPoint.X);
                    }
                }
                if (MouseIsInRightEdge)
                {
                    if (MouseIsInTopEdge)
                    {
                        lControl.Width = (lEvent.X - mCursorStartPoint.X) + mControlStartSize.Width;
                        lControl.Height -= (lEvent.Y - mCursorStartPoint.Y);
                        lControl.Top += (lEvent.Y - mCursorStartPoint.Y);

                    }
                    else if (MouseIsInBottomEdge)
                    {
                        lControl.Width = (lEvent.X - mCursorStartPoint.X) + mControlStartSize.Width;
                        lControl.Height = (lEvent.Y - mCursorStartPoint.Y) + mControlStartSize.Height;
                    }
                    else
                    {
                        lControl.Width = (lEvent.X - mCursorStartPoint.X) + mControlStartSize.Width;
                    }
                }
                else if (MouseIsInTopEdge)
                {
                    lControl.Height -= (lEvent.Y - mCursorStartPoint.Y);
                    lControl.Top += (lEvent.Y - mCursorStartPoint.Y);
                }
                else if (MouseIsInBottomEdge)
                {
                    lControl.Height = (lEvent.Y - mCursorStartPoint.Y) + mControlStartSize.Height;
                }
                else
                {
                    StopDragOrResizing(lControl);
                }
            }
            else if (mMoving)
            {
                mMoveIsInternal = !mMoveIsInternal;
                if (!mMoveIsInternal)
                {
                    int x = (lEvent.X - mCursorStartPoint.X) + lControl.Left;
                    int y = (lEvent.Y - mCursorStartPoint.Y) + lControl.Top;
                    lControl.Location = new Point(x, y);
                }
            }
        }

        private void StopDragOrResizing(Control lControl)
        {
            if (mToggleButtonState == ToggleButtonState.Collapsed || !AllowResize)
                return;

            mResizing = false;
            mMoving = false;
            lControl.Capture = false;
            UpdateMouseCursor(lControl);
        }

        /// <summary>
        /// Method to initialze the control initial theme
        /// </summary>
        private void InitializeDefaultControlsTheme()
        {
            //Register for resizing
            
            Register(this);

            //initialize toggle button position
            mToggleButtonPosition = CollasibleButtonPosition.Right;

            //initialize toggle button state
            mToggleButtonState = ToggleButtonState.Expanded;

            //Set the orientation text
            mOrientationText = this.Name;
            base.Font = new System.Drawing.Font("Microsoft Sans Serif", 12F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(0)));

            //Set the default properties of the Controls
            Size = new Size(300, 150);
            BackColor = Color.White;
            BorderStyle = BorderStyle.None;
            Padding = new Padding(0, 0, 0, 0);
            mControlsSize = this.Size;

            //Set the image button style
            ctrToggleButtonPicturebox.Size = new Size(22, 22);
            ctrToggleButtonPicturebox.SizeMode = PictureBoxSizeMode.CenterImage;
            ctrToggleButtonPicturebox.BackColor = this.BackColor;
            ctrToggleButtonPicturebox.BorderStyle = System.Windows.Forms.BorderStyle.None;
            ctrToggleButtonPicturebox.Location = GetToggleButtonLocation();
           
            ctrToggleButtonPicturebox.SizeMode = PictureBoxSizeMode.CenterImage;
            BindImagesToToggleButton();
            ctrToggleButtonPicturebox.MouseHover += OnMouseHoverAndEnterToPictureBox;
            ctrToggleButtonPicturebox.MouseEnter += OnMouseHoverAndEnterToPictureBox;
            ctrToggleButtonPicturebox.MouseLeave += OnButtonMosueLeave;
            ctrToggleButtonPicturebox.MouseClick += OnMouseClick;                     
        }

       
        /// <summary>
        /// Method to get the Toggle button location based on toggle button State
        /// </summary>
        /// <returns></returns>
        private Point GetToggleButtonLocation()
        {
            Point lPoint = new Point();
            switch (mToggleButtonState)
            {
                case ToggleButtonState.Expanded:
                    if(mToggleButtonPosition == CollasibleButtonPosition.Right)
                        lPoint = new Point(this.Width - ctrToggleButtonPicturebox.Width - BorderThickness.RightBorderThickness , 1);
                    else
                        lPoint = new Point(1, 1);
                    break;
                case ToggleButtonState.Collapsed:
                    lPoint = new Point(1, 1);
                    break;
            }
            return lPoint;
        }

        /// <summary>
        /// This methoe will execute when mouse click will happen on Toggle Image button.
        /// which will change the toggle state and image of the control
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OnMouseClick(object sender, MouseEventArgs e)
        {
            OnToggleButtonMouseClick();            
        }

        /// <summary>
        /// Metod to execute when toggle button click event
        /// </summary>
        private void OnToggleButtonMouseClick()
        {
            switch (mToggleButtonState)
            {
                case ToggleButtonState.Expanded:
                    this.mToggleButtonState = ToggleButtonState.Collapsed;
                    for (int lint = 0; lint < this.Controls.Count; lint++)
                    {
                        if (this.Controls[lint].Name == "ctrToggleButtonPicturebox")
                            continue;

                        this.Controls[lint].Visible = false;
                    }
                    break;

                case ToggleButtonState.Collapsed:
                    this.mToggleButtonState = ToggleButtonState.Expanded;
                    for (int lint = 0; lint < this.Controls.Count; lint++)
                    {
                        this.Controls[lint].Visible = true;
                    }
                    break;
            }

            //First bind the toggle images
            BindImagesToToggleButton();

            //Resize the control height
            ResizeControlOnToggleButtonStateChange();

            //Invalidate the control to draw the resize border            
            Invalidate();
        }

        /// <summary>
        /// Method to resize the control panel when the controls toggle state change.
        /// </summary>
        private void ResizeControlOnToggleButtonStateChange()
        {
            //First cache the current control height
            switch (mToggleButtonState)
            {
                case ToggleButtonState.Expanded:
                    this.Width = mControlsSize.Width;
                    break;

                case ToggleButtonState.Collapsed:
                    mControlsSize = this.Size;
                    this.Width = ctrToggleButtonPicturebox.Width +2;
                    break;
            }
            ctrToggleButtonPicturebox.Location = GetToggleButtonLocation();
        }

        /// <summary>
        /// Method will execute on toggle button click
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OnButtonMosueLeave(object sender, EventArgs e)
        {
            BindImagesToToggleButton();
        }

        /// <summary>
        /// Method to bind images to toggle button according to the toggle state
        /// </summary>
        private void BindImagesToToggleButton()
        {
            switch (mToggleButtonState)
            {
                case ToggleButtonState.Expanded:
                    ctrToggleButtonPicturebox.Image = global::Eka.Inheritance.MDIControls.Properties.Resources.LeftArrowWhite;
                    break;

                case ToggleButtonState.Collapsed:
                    ctrToggleButtonPicturebox.Image = global::Eka.Inheritance.MDIControls.Properties.Resources.RithtArrowWhite;
                    break;
            }
        }

        /// <summary>
        /// Method to call on mouse hover and enter event for Toggle image button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OnMouseHoverAndEnterToPictureBox(object sender, EventArgs e)
        {
            BindMouseHoverAndEnterImageToButton();
        }

        /// <summary>
        /// Method to call on toggle button mosue hover and enter button
        /// </summary>
        private void BindMouseHoverAndEnterImageToButton()
        {
            switch (mToggleButtonState)
            {
                case ToggleButtonState.Expanded:
                    ctrToggleButtonPicturebox.Image = global::Eka.Inheritance.MDIControls.Properties.Resources.LeftArrowOrange;
                    break;

                case ToggleButtonState.Collapsed:
                    ctrToggleButtonPicturebox.Image = global::Eka.Inheritance.MDIControls.Properties.Resources.RithtArrowOrange;
                    break;
            }
        }

        #endregion

        #region overridden method
        /// <summary>
        /// override the OnResize method and call the Invalidate to draw the border properly
        /// </summary>
        /// <param name="e"></param>
        protected override void OnResize(EventArgs e)
        {
            base.OnResize(e);
            ctrToggleButtonPicturebox.Location = GetToggleButtonLocation();
            Invalidate();
        }

        /// <summary>
        /// Paint method for drawing the Border line
        /// </summary>
        /// <param name="e"></param>
        protected override void OnPaint(PaintEventArgs e)
        {
            base.OnPaint(e);
            switch (mToggleButtonState)
            {
                case ToggleButtonState.Expanded:
                    if (mDrawBorder)
                    {
                        ControlPaint.DrawBorder(e.Graphics, ClientRectangle,
                                mBorderColor, BorderThickness.LeftBorderThickness , mButtonBorderStyle,
                                mBorderColor, BorderThickness.TopBorderThickness, mButtonBorderStyle,
                                mRightBorderColor, BorderThickness.RightBorderThickness, mButtonBorderStyle,
                                mBorderColor, BorderThickness.BottomBorderThickness, mButtonBorderStyle);
                    }
                    else
                    {
                        ControlPaint.DrawBorder(e.Graphics, ClientRectangle,
                                Color.Transparent, BorderThickness.LeftBorderThickness, mButtonBorderStyle,
                                Color.Transparent, BorderThickness.TopBorderThickness, mButtonBorderStyle,
                                Color.Transparent, BorderThickness.RightBorderThickness, mButtonBorderStyle,
                                Color.Transparent, BorderThickness.BottomBorderThickness, mButtonBorderStyle);
                    }
                    break;

                case ToggleButtonState.Collapsed:
                    if (mDrawBorder)
                    {
                        ControlPaint.DrawBorder(e.Graphics, new Rectangle(new Point(0, 0), new Size(ctrToggleButtonPicturebox.Width + 2, this.Height)),
                                    mBorderColor, 1, mButtonBorderStyle,
                                    mBorderColor, 1, mButtonBorderStyle,
                                   mRightBorderColor, 1, mButtonBorderStyle,
                                    mBorderColor, 1, mButtonBorderStyle);
                    }
                    else
                    {
                        ControlPaint.DrawBorder(e.Graphics, new Rectangle(new Point(0, 0), new Size(ctrToggleButtonPicturebox.Width + 2, this.Height)),
                                    Color.Transparent, 1, mButtonBorderStyle,
                                    Color.Transparent, 1, mButtonBorderStyle,
                                    Color.Transparent, 1, mButtonBorderStyle,
                                    Color.Transparent, 1, mButtonBorderStyle);
                    }

                    if (mDrawOrientationText)
                        DrawText(e.Graphics, mOrientationText);
                    else
                        DrawText(e.Graphics, "");
                    break;
            }
        }
        /// <summary>
        /// override this method to make the Cursor normal
        /// </summary>
        /// <param name="e"></param>
        protected override void OnMouseLeave(EventArgs e)
        {
            base.OnMouseLeave(e);
            this.Cursor = Cursors.Arrow;
        }

        private void DrawText(Graphics lGraphics, string lTextToDraw)
        {
            SolidBrush drawBrush = new System.Drawing.SolidBrush(mOrientationFontColor);
            float x = 1.0f;
            float y = ctrToggleButtonPicturebox.Location.X +25 + ctrToggleButtonPicturebox.Location.Y + 2;
            System.Drawing.StringFormat drawFormat = new System.Drawing.StringFormat(StringFormatFlags.DirectionVertical);
            lGraphics.DrawString(lTextToDraw, this.Font, drawBrush, x, y, drawFormat);
            drawBrush.Dispose();
        }
        #endregion

        /// <summary>
        /// 
        /// </summary>
        /// <param name="e"></param>
        protected override void OnHandleCreated(EventArgs e)
        {
            base.OnHandleCreated(e);


            UpdateExtendedStyles();
            if (!IsWinXP)
                SendMessage(Handle, TVM_SETBKCOLOR, IntPtr.Zero, (IntPtr)ColorTranslator.ToWin32(BackColor));
        }

        #region Private Method New
        private void UpdateExtendedStyles()
        {
            int Style = 0;

            if (DoubleBuffered)
                Style |= TVS_EX_DOUBLEBUFFER;

            if (Style != 0)
                SendMessage(Handle, TVM_SETEXTENDEDSTYLE, (IntPtr)TVS_EX_DOUBLEBUFFER, (IntPtr)Style);
        }

        private static bool IsWinVista
        {
            get
            {
                OperatingSystem OS = Environment.OSVersion;
                return (OS.Platform == PlatformID.Win32NT) && (OS.Version.Major >= 6);
            }
        }

        private static bool IsWinXP
        {
            get
            {
                OperatingSystem OS = Environment.OSVersion;
                return (OS.Platform == PlatformID.Win32NT) &&
                    ((OS.Version.Major > 5) || ((OS.Version.Major == 5) && (OS.Version.Minor == 1)));
            }
        }
        #endregion
    }
}
